@@ -48,7 +48,7 @@ Block::Block(Device *dev)
                 if (!nodeElem.isNull() && nodeElem.hasAttribute(QStringLiteral("name"))) {
                     const QString udi = QStringLiteral(UD2_DBUS_PATH_BLOCKDEVICES) + QLatin1Char('/') + nodeElem.attribute(QStringLiteral("name"));
 
-                    Device device(udi);
+                    Device device(dev->manager(), udi);
                     if (device.drivePath() == dev->udi()) {
                         m_devNum = device.prop(QStringLiteral("DeviceNumber")).toULongLong();
                         m_devFile = QFile::decodeName(device.prop(QStringLiteral("Device")).toByteArray());
