@@ -8,12 +8,14 @@
 #include "udisksstorageaccess.h"
 #include "udisks2.h"
 #include "udisks_debug.h"
+#include "udisksutils.h"
 
 #include <QDBusConnection>
 #include <QDBusInterface>
 #include <QDBusMetaType>
 #include <QDir>
 #include <QGuiApplication>
+#include <QMetaObject>
 #include <QWindow>
 
 #include <config-solid.h>
@@ -49,6 +51,7 @@ StorageAccess::StorageAccess(Device *device)
     : DeviceInterface(device)
     , m_setupInProgress(false)
     , m_teardownInProgress(false)
+    , m_checkInProgress(false)
     , m_repairInProgress(false)
     , m_passphraseRequested(false)
 {
@@ -59,7 +62,7 @@ StorageAccess::StorageAccess(Device *device)
 
     // Delay connecting to DBus signals to avoid the related time penalty
     // in hot paths such as predicate matching
-    QTimer::singleShot(0, this, SLOT(connectDBusSignals()));
+    QMetaObject::invokeMethod(this, &StorageAccess::connectDBusSignals, Qt::QueuedConnection);
 }
 
 StorageAccess::~StorageAccess()
@@ -72,6 +75,8 @@ void StorageAccess::connectDBusSignals()
 
     m_device->registerAction(QStringLiteral("teardown"), this, SLOT(slotTeardownRequested()), SLOT(slotTeardownDone(int, QString)));
 
+    m_device->registerAction(QStringLiteral("check"), this, SLOT(slotCheckRequested()), SLOT(slotCheckDone(int, QString)));
+
     m_device->registerAction(QStringLiteral("repair"), this, SLOT(slotRepairRequested()), SLOT(slotRepairDone(int, QString)));
 }
 
@@ -88,7 +93,7 @@ bool StorageAccess::isAccessible() const
         if (path.isEmpty() || path == QLatin1String("/")) {
             return false;
         }
-        Device holderDevice(path);
+        Device holderDevice(m_device->manager(), path);
         return holderDevice.isMounted();
     }
 
@@ -124,19 +129,18 @@ bool StorageAccess::canCheck() const
 
 bool StorageAccess::check()
 {
-    if (m_setupInProgress || m_teardownInProgress) {
+    if (m_setupInProgress || m_teardownInProgress || m_checkInProgress || m_repairInProgress) {
         return false;
     }
+    m_checkInProgress = true;
+    m_device->broadcastActionRequested(QStringLiteral("check"));
 
     const auto path = dbusPath();
     auto c = QDBusConnection::systemBus();
     auto msg = QDBusMessage::createMethodCall(QStringLiteral(UD2_DBUS_SERVICE), path, QStringLiteral(UD2_DBUS_INTERFACE_FILESYSTEM), QStringLiteral("Check"));
-    QVariantMap options;
-    msg << options;
-    QDBusReply<bool> r = c.call(msg);
-    bool ret = r.isValid() && r.value();
-    qCDebug(UDISKS2) << Q_FUNC_INFO << path << ret;
-    return ret;
+    msg << QVariantMap{};
+
+    return c.callWithCallback(msg, this, SLOT(slotDBusReply(QDBusMessage)), SLOT(slotDBusError(QDBusError)));
 }
 
 bool StorageAccess::canRepair() const
@@ -161,7 +165,7 @@ bool StorageAccess::canRepair() const
 
 bool StorageAccess::repair()
 {
-    if (m_teardownInProgress || m_setupInProgress || m_repairInProgress) {
+    if (m_teardownInProgress || m_setupInProgress || m_checkInProgress || m_repairInProgress) {
         return false;
     }
     m_repairInProgress = true;
@@ -191,7 +195,7 @@ static QString baseMountPoint(const QByteArray &dev)
             struct libmnt_iter *itr = mnt_new_iter(MNT_ITER_BACKWARD);
             struct libmnt_fs *fs;
 
-            const QByteArray devicePath = dev.endsWith('\x00') ? dev.chopped(1) : dev;
+            const QByteArray devicePath = Utils::sanitizeValue(dev);
 
             while (mnt_table_next_fs(table, itr, &fs) == 0) {
                 if (mnt_fs_get_srcpath(fs) == devicePath //
@@ -221,14 +225,10 @@ QString StorageAccess::filePath() const
         if (path.isEmpty() || path == QLatin1String("/")) {
             return QString();
         }
-        Device holderDevice(path);
+        Device holderDevice(m_device->manager(), path);
         const auto mntPoints = qdbus_cast<QByteArrayList>(holderDevice.prop(QStringLiteral("MountPoints")));
         if (!mntPoints.isEmpty()) {
-            QByteArray first = mntPoints.first();
-            if (first.endsWith('\x00')) {
-                first.chop(1);
-            }
-            return QFile::decodeName(first); // FIXME Solid doesn't support multiple mount points
+            return QFile::decodeName(Utils::sanitizeValue(mntPoints.first())); // FIXME Solid doesn't support multiple mount points
         } else {
             return QString();
         }
@@ -239,11 +239,7 @@ QString StorageAccess::filePath() const
         return {};
     }
 
-    QByteArray first = mntPoints.first();
-    if (first.endsWith('\x00')) {
-        first.chop(1);
-    }
-    const QString potentialMountPoint = QFile::decodeName(first);
+    const QString potentialMountPoint = QFile::decodeName(Utils::sanitizeValue(mntPoints.first()));
 
     if (mntPoints.size() == 1) {
         return potentialMountPoint;
@@ -276,7 +272,7 @@ bool StorageAccess::isIgnored() const
 
 bool StorageAccess::setup()
 {
-    if (m_teardownInProgress || m_setupInProgress || m_repairInProgress) {
+    if (m_teardownInProgress || m_setupInProgress || m_checkInProgress || m_repairInProgress) {
         return false;
     }
     m_setupInProgress = true;
@@ -291,7 +287,7 @@ bool StorageAccess::setup()
 
 bool StorageAccess::teardown()
 {
-    if (m_teardownInProgress || m_setupInProgress || m_repairInProgress) {
+    if (m_teardownInProgress || m_setupInProgress || m_checkInProgress || m_repairInProgress) {
         return false;
     }
     m_teardownInProgress = true;
@@ -315,14 +311,13 @@ void StorageAccess::checkAccessibility()
     }
 }
 
-void StorageAccess::slotDBusReply(const QDBusMessage & /*reply*/)
+void StorageAccess::slotDBusReply(const QDBusMessage &reply)
 {
     if (m_setupInProgress) {
         if (isLuksDevice() && !isAccessible()) { // unlocked device, now mount it
             mount();
         } else { // Don't broadcast setupDone unless the setup is really done. (Fix kde#271156)
             m_setupInProgress = false;
-            m_device->invalidateCache();
             m_device->broadcastActionDone(QStringLiteral("setup"));
 
             checkAccessibility();
@@ -338,7 +333,7 @@ void StorageAccess::slotDBusReply(const QDBusMessage & /*reply*/)
             // try to "eject" (aka safely remove) from the (parent) drive, e.g. SD card from a reader
             QString drivePath = m_device->drivePath();
             if (!drivePath.isEmpty() || drivePath != QStringLiteral("/")) {
-                Device drive(drivePath);
+                Device drive(m_device->manager(), drivePath);
                 QDBusConnection c = QDBusConnection::systemBus();
 
                 if (drive.prop(QStringLiteral("MediaRemovable")).toBool() //
@@ -363,11 +358,19 @@ void StorageAccess::slotDBusReply(const QDBusMessage & /*reply*/)
             }
 
             m_teardownInProgress = false;
-            m_device->invalidateCache();
             m_device->broadcastActionDone(QStringLiteral("teardown"));
 
             checkAccessibility();
         }
+    } else if (m_checkInProgress) {
+        QDBusReply<bool> r = reply;
+        qCDebug(UDISKS2) << "Check reply received " << m_device->udi() << r;
+        m_checkInProgress = false;
+        if (r.isValid()) {
+            m_device->broadcastActionDone(QStringLiteral("check"), Solid::NoError, QString::number(r.value()));
+        } else {
+            m_device->broadcastActionDone(QStringLiteral("check"), Solid::OperationFailed, reply.errorMessage());
+        }
     } else if (m_repairInProgress) {
         qCDebug(UDISKS2) << "Successfully repaired " << m_device->udi();
         m_repairInProgress = false;
@@ -392,6 +395,11 @@ void StorageAccess::slotDBusError(const QDBusError &error)
                                       m_device->errorToSolidError(error.name()),
                                       m_device->errorToString(error.name()) + QStringLiteral(": ") + error.message());
         checkAccessibility();
+    } else if (m_checkInProgress) {
+        m_checkInProgress = false;
+        m_device->broadcastActionDone(QStringLiteral("check"),
+                                      m_device->errorToSolidError(error.name()),
+                                      m_device->errorToString(error.name()) + QStringLiteral(": ") + error.message());
     } else if (m_repairInProgress) {
         m_repairInProgress = false;
         m_device->broadcastActionDone(QStringLiteral("repair"),
@@ -428,6 +436,18 @@ void StorageAccess::slotTeardownDone(int error, const QString &errorString)
     Q_EMIT teardownDone(static_cast<Solid::ErrorType>(error), errorString, m_device->udi());
 }
 
+void StorageAccess::slotCheckRequested()
+{
+    m_checkInProgress = true;
+    Q_EMIT checkRequested(m_device->udi());
+}
+
+void StorageAccess::slotCheckDone(int error, const QString &errorString)
+{
+    m_checkInProgress = false;
+    Q_EMIT checkDone(static_cast<Solid::ErrorType>(error), errorString, m_device->udi());
+}
+
 void StorageAccess::slotRepairRequested()
 {
     m_repairInProgress = true;
