@@ -86,6 +86,7 @@ bool UPowerDevice::queryDeviceInterface(const Solid::DeviceInterface::Type &type
         case UP_DEVICE_KIND_HEADPHONES:
         case UP_DEVICE_KIND_BLUETOOTH_GENERIC:
         case UP_DEVICE_KIND_TOUCHPAD:
+        case UP_DEVICE_KIND_WEARABLE:
             return true;
         case UP_DEVICE_KIND_UNKNOWN:
             // There is currently no "Bluetooth battery" type, so check if it comes from Bluez
@@ -99,8 +100,9 @@ bool UPowerDevice::queryDeviceInterface(const Solid::DeviceInterface::Type &type
         case UP_DEVICE_KIND_COMPUTER:
         case UP_DEVICE_KIND_LAST:
             return false;
+        default:
+            Q_FALLTHROUGH();
         }
-        Q_FALLTHROUGH();
     default:
         return false;
     }
@@ -114,14 +116,15 @@ QStringList UPowerDevice::emblems() const
 QString UPowerDevice::description() const
 {
     if (queryDeviceInterface(Solid::DeviceInterface::Battery)) {
-        return tr("%1 Battery", "%1 is battery technology").arg(batteryTechnology());
-    } else {
-        QString result = prop(QStringLiteral("Model")).toString();
-        if (result.isEmpty()) {
-            return vendor();
+        if (!batteryTechnology().isEmpty()) {
+            return tr("%1 Battery", "%1 is battery technology").arg(batteryTechnology());
         }
-        return result;
     }
+    QString result = prop(QStringLiteral("Model")).toString();
+    if (result.isEmpty()) {
+        return vendor();
+    }
+    return result;
 }
 
 QString UPowerDevice::batteryTechnology() const
@@ -129,7 +132,7 @@ QString UPowerDevice::batteryTechnology() const
     const UpDeviceTechnology tech = static_cast<UpDeviceTechnology>(prop(QStringLiteral("Technology")).toUInt());
     switch (tech) {
     case UP_DEVICE_TECHNOLOGY_UNKNOWN:
-        return tr("Unknown", "battery technology");
+        return QString();
     case UP_DEVICE_TECHNOLOGY_LITHIUM_ION:
         return tr("Lithium Ion", "battery technology");
     case UP_DEVICE_TECHNOLOGY_LITHIUM_POLYMER:
@@ -143,9 +146,9 @@ QString UPowerDevice::batteryTechnology() const
     case UP_DEVICE_TECHNOLOGY_NICKEL_METAL_HYDRIDE:
         return tr("Nickel Metal Hydride", "battery technology");
     case UP_DEVICE_TECHNOLOGY_LAST:
-        return tr("Unknown", "battery technology");
+        return QString();
     }
-    return tr("Unknown", "battery technology");
+    return QString();
 }
 
 QString UPowerDevice::icon() const
